randn('state', 1);

% First, define a base autoencoder the algorithm will run on:
D = 7;           % Data dimensionality
%K = [3, 5];     % # hiddens in each layer, fail final check with a few hiddens.
K = [13, 21];    % # hiddens in each layer
L = length(K);   % # hidden layers
W = cell(1, L);  % Hidden layer weights
bb = cell(1, L); % Hidden layer biases
W{1} = randn(K(1), D)/10;
bb{1} = randn(K(1), 1)/10;
for ll = 2:L
    W{ll} = randn(K(ll), K(ll-1))/10;
    bb{ll} = randn(K(ll), 1)/10;
end
V = randn(D, K(end))/10; % DxK(L) output weights
cc = randn(D, 1)/10; % output biases

% Check gradients g_X against finite difference approximations d_X:
[ell, g_b, g_W, g_c, g_V] = made_loss(bb, W, cc, V);
d_b = g_b;
d_W = g_W;
[max_errors, d_b{:}, d_W{:}, d_c, d_V] = var_checkgrad(@made_loss_check_wrapper, 1e-3, bb{:}, W{:}, cc, V);
fprintf('Checking gradients against finite differences:\n');
max_error = max(max_errors)
assert(max_error < 1e-5);

% Check probabilities are normalized:
fprintf('\nChecking probabilities sum to 1:\n');
N = 2^D;
log_probs = zeros(1, N);
for nn = 1:N
    xx = bitget(nn, 1:D);
    neg_log_probs(nn) = made_loss(bb, W, cc, V, xx);
end
Z = exp(logsumexp(-neg_log_probs))
assert(abs(Z - 1) < 1e-9);

% Check masking works as advertised. If change dimension d_flip, the output for
% d_flip and all previous variables in the ordering should not change. The other
% outputs will all change if there are paths through to them (which there will
% be if there are lots of hiddens).
ordering = randperm(D)';
xx1 = rand(D, 1) < 0.5;
d_flip = 3;
xx2 = xx1; xx2(d_flip) = 1 - xx2(d_flip);
output_x_hat = 1;
x_hat1 = made_loss(bb, W, cc, V, xx1, ordering, output_x_hat);
x_hat2 = made_loss(bb, W, cc, V, xx2, ordering, output_x_hat);
if isequal(x_hat1==x_hat2, ordering<=ordering(d_flip))
    fprintf('\nGreat: When I changed a variable, all variables later in ordering changed\n');
elseif any((x_hat1==x_hat2) .* (ordering>ordering(d_flip)))
    fprintf('\nWhen I changed a variable, not all outputs that could change did. Could use more hiddens.\n');
end
if any((x_hat1~=x_hat2) .* (ordering<=ordering(d_flip)))
    error('Fatal: When I changed a variable, some outputs changed that must not. Bug.\n');
end

