function [ell, delta_b, delta_W, delta_c, delta_V] = made_loss(bb, W, cc, V, xx, ordering, output_x_hat)
%MADE_LOSS rewriting of MADE pseudo-code for checking/tutorial purposes
%
% May be a useful supplement in case anyone finds the pseudo-code ambiguous.
% (Simpler than looking at large, real codebase.)

%% Some setup before I can start rewriting the pseudo-code

% Clamp random number generator, so can check consistency of gradients with
% multiple calls to this function:
rand('twister', 10);

% First, complete the definition of a base autoencoder the algorithm will run on:
D = size(V, 1); % Data dimension
L = length(W); % # hidden layers
K = zeros(1, L);
for ll = 1:L
    K(ll) = size(W{ll}, 1); % # hiddens in each layer
end
sigm = @(a) 1./(1 + exp(-a)); % output activation function (must be sigmoidal in [0,1])
gg = @(a) log(1 + exp(a)); % hidden activation function (can change, but must change gprime to match)
gprime = sigm; % gg's derivative

% Input for this SGD round:
if nargin < 5
    xx = rand(D, 1) < 0.5;
else
    assert(isvector(xx)); xx = xx(:);
end


%% Fairly close rewriting of the pseudo-code starts here

% Sampling the m^l vectors
mm = cell(1, L+1); % mm{l+1} stores m^l, l in 0,1,...,L. % CARE: include the +1
if nargin < 6
    mm{(0)+1} = randperm(D)';
else
    mm{(0)+1} = ordering; % not in pseudo-code. Added this hook so I can test the code outside this routine.
end
for ll = 1:L
    min_m = min(mm{(ll-1)+1});
    mm{(ll)+1} = randi([min_m, D-1], K(ll), 1);
end

% Constructing masks for each layer
one_op = @(op, a, b) bsxfun(op, a(:), b(:)'); % To match definition of 1_{a op b} in paper
MW = cell(1, L); % ML{l} stores M^W^l
% For one layer, MW{1} is K(1)xD
% For ll>1, MW{ll} is K(ll) x K(ll-1)
for ll = 1:L
    MW{ll} = one_op(@ge, mm{(ll)+1}, mm{(ll-1)+1});
    assert(isequal(size(MW{ll}), size(W{ll})));
end
MV = one_op(@gt, mm{(0)+1}, mm{(L)+1}); % D x K(L)
assert(isequal(size(MV), size(V)));

% Computing $p(x)$
hh = cell(1, L+1); % hh{l+1} stores h^l, where h^0 is input. CARE: like mm, make sure to include the +1
hh{(0)+1} = xx;
for ll = 1:L
    hh{(ll)+1} = gg(bb{ll} + (W{ll}.*MW{ll})*hh{(ll-1)+1}); % K(ll)x1
    assert(isequal(size(hh{(ll)+1}), [K(ll), 1]));
end
x_hat = sigm(cc + (V.*MV)*hh{(L)+1}); % Dx1
log_px = sum(xx.*log(x_hat) + (1-xx).*log(1-x_hat)); % 1x1
p_x = exp(log_px); % 1x1
% In real code wouldn't take exp, and would just return -log_px:
ell = -log_px;

% Computing gradients of -log p(x)
tmp = x_hat - xx;
delta_c = tmp;
delta_V = (tmp * hh{(L)+1}') .* MV;
tmp = (tmp'*(V.*MV))';
for ll = L:-1:1
    tmp = tmp .* gprime(bb{ll} + (W{ll}.*MW{ll})*hh{(ll-1)+1});
    delta_b{ll} = tmp;
    delta_W{ll} = (tmp * hh{(ll-1)+1}') .* MW{ll};
    tmp = (tmp'*(W{ll}.*MW{ll}))';
end


% The final optional argument is a flag used for testing, so I can inspect all
% autoencoder outputs:
if (nargin > 6) && output_x_hat
    ell = x_hat;
end

